;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-*-mode:scheme-*-
;;;                                                                       ;;
;;;                            [SOMEBODY]                                 ;;
;;;                         Copyright (c) 2000                            ;;
;;;                        All Rights Reserved.                           ;;
;;;                                                                       ;;
;;;  Distribution policy                                                  ;;
;;;     [CHOOSE ONE OF]                                                   ;;
;;;     Free for any use                                                  ;;
;;;     Free for non commercial use                                       ;;
;;;     something else                                                    ;;
;;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  An example diphone voice
;;;
;;;  Authors: [The people who did the work]
;;;

;;; Try to find out where we are
(if (assoc 'itc_kh_po_diphone voice-locations)
    (defvar itc_kh_po_dir 
      (cdr (assoc 'itc_kh_po_diphone voice-locations)))
    ;;; Not installed in Festival yet so assume running in place
    (defvar itc_kh_po_dir (pwd)))

(if (not (probe_file (path-append itc_kh_po_dir "festvox/")))
    (begin
     (format stderr "itc_kh_po: Can't find voice scm files they are not in\n")
     (format stderr "   %s\n" (path-append itc_kh_po_dir "festvox/"))
     (format stderr "   Either the voice isn't linked into Festival\n")
     (format stderr "   or you are starting festival in the wrong directory\n")
     (error)))

;;;  Add the directory contains general voice stuff to load-path
(set! load-path (cons (path-append itc_kh_po_dir "festvox/") load-path))

;;; Voice specific parameter are defined in each of the following
;;; files
(require 'itc_kh_po_phoneset)
(require 'itc_kh_po_tokenizer)
(require 'itc_kh_po_tagger)
(require 'itc_kh_po_lexicon)
(require 'itc_kh_po_phrasing)

(require 'itc_kh_po_pause)
(require 'itc_kh_po_utt_break)

(require 'itc_kh_po_intonation)
(require 'itc_kh_po_duration)
(require 'itc_kh_po_f0model)
(require 'itc_kh_po_other)
;; ... and others as required

;;;  Ensure we have a festival with the right diphone support compiled in
(require_module 'UniSyn)

(set! itc_kh_po_lpc_sep 
      (list
       '(name "itc_kh_po_lpc_sep")
       (list 'index_file (path-append itc_kh_po_dir "dic/podiph.est"))
       '(grouped "false")
       (list 'coef_dir (path-append itc_kh_po_dir "lpc"))
       (list 'sig_dir  (path-append itc_kh_po_dir "lpc"))
       '(coef_ext ".lpc")
       '(sig_ext ".res")
       (list 'default_diphone 
	     (string-append
	      (car (cadr (car (PhoneSet.description '(silences)))))
	      "-"
	      (car (cadr (car (PhoneSet.description '(silences)))))))))

(set! itc_kh_po_lpc_group 
      (list
       '(name "po_lpc_group")
       (list 'index_file 
	     (path-append itc_kh_po_dir "group/polpc.group"))
       '(grouped "true")
       (list 'default_diphone 
	     (string-append
	      (car (cadr (car (PhoneSet.description '(silences)))))
	      "-"
	      (car (cadr (car (PhoneSet.description '(silences)))))))))

;; Go ahead and set up the diphone db
;(set! itc_kh_po_db_name (us_diphone_init itc_kh_po_lpc_sep))

;; Once you've built the group file you can comment out the above and
;; uncomment the following.
(set! itc_kh_po_db_name (us_diphone_init itc_kh_po_lpc_group))

(define (itc_kh_po_diphone_fix utt)
"(itc_kh_po_diphone_fix UTT)
Map phones to phonological variants if the diphone database supports
them."
  (mapcar
   (lambda (s)
     (let ((name (item.name s)))
       (itc_kh_po_diphone_fix_phone_name utt s)
       ))
   (utt.relation.items utt 'Segment))
  utt)

(define (itc_kh_po_diphone_fix_phone_name utt seg))
"(itc_kh_po_fix_phone_name UTT SEG)
Add the feature diphone_phone_name to given segment with the appropriate
name for constructing a diphone.  Basically adds _ if either side is part
of the same consonant cluster, adds $ either side if in different
syllable for preceding/succeeding vowel syllable."
;;  (let ((name (item.name seg)))
;;    (cond
;;     ((string-equal name "pau") t)
;;     ((string-equal "-" (item.feat seg 'ph_vc))
;;      (if (and (member_string name '(r w y l))
;;	       (member_string (item.feat seg "p.name") '(p t k b d g))
;;	       (item.relation.prev seg "SylStructure"))
;;	  (item.set_feat seg "us_diphone_right" (format nil "_%s" name)))
;;      (if (and (member_string name '(w y l m n p t k))
;;	       (string-equal (item.feat seg "p.name") 's)
;;	       (item.relation.prev seg "SylStructure"))
;;	  (item.set_feat seg "us_diphone_right" (format nil "_%s" name)))
;;      (if (and (string-equal name 's)
;;	       (member_string (item.feat seg "n.name") '(w y l m n p t k))
;;	       (item.relation.next seg "SylStructure"))
;;	  (item.set_feat seg "us_diphone_left" (format nil "%s_" name)))
;;      (if (and (string-equal name 'hh)
;;	       (string-equal (item.feat seg "n.name") 'y))
;;	  (item.set_feat seg "us_diphone_left" (format nil "%s_" name)))
;;      (if (and (string-equal name 'y)
;;	       (string-equal (item.feat seg "p.name") 'hh))
;;	  (item.set_feat seg "us_diphone_right" (format nil "_%s" name)))
;;      (if (and (member_string name '(p t k b d g))
;;	       (member_string (item.feat seg "n.name") '(r w y l))
;;	       (item.relation.next seg "SylStructure"))
;;	  (item.set_feat seg "us_diphone_left" (format nil "%s_" name)))
;;      )
;;     ((string-equal "ah" (item.name seg))
;;      (item.set_feat seg "us_diphone" "aa"))
;;
;;   )))

(define (itc_kh_po_voice_reset)
  "(itc_kh_po_voice_reset)
Reset global variables back to previous voice."
  (itc_kh_po::reset_phoneset)
  (itc_kh_po::reset_tokenizer)
  (itc_kh_po::reset_tagger)
  (itc_kh_po::reset_lexicon)
  (itc_kh_po::reset_phrasing)
  (itc_kh_po::reset_intonation)
  (itc_kh_po::reset_duration)
  (itc_kh_po::reset_f0model)
  (itc_kh_po::reset_other)
)

;;;  Full voice definition 
(define (voice_itc_kh_po_diphone)
"(voice_itc_kh_po_diphone)
Set speaker to po in us from itc."
  ;; Select appropriate phone set
  (itc_kh_po::select_phoneset)

  ;; Select appropriate tokenization
  (itc_kh_po::select_tokenizer)

  ;; For part of speech tagging
  (itc_kh_po::select_tagger)

  (itc_kh_po::select_lexicon)

  (itc_kh_po::select_phrasing)

  (itc_kh_po::select_intonation)

  (itc_kh_po::select_duration)

  (itc_kh_po::select_f0model)

  ; If we don't have the default English voice installed (e.g., on
  ; Windows), then we must initialize a few parameters to get Festival
  ; to start up properly. Snarfed from Rhys Jones' change in Welsh 
  ; Festival.
  (Parameter.set 'Token_Method 'Token_Any)
  (Parameter.set 'POS_Method Classic_POS)
  (Parameter.set 'Phrasify_Method Classic_Phrasify)
  (Parameter.set 'Word_Method Classic_Word)
  (Parameter.set 'Pause_Method Classic_Pauses)
  (Parameter.set 'PostLex_Method Classic_PostLex)

  ;; Waveform synthesizer: UniSyn diphones
  (set! UniSyn_module_hooks (list itc_kh_po_diphone_fix))
  (set! us_abs_offset 0.0)
  (set! window_factor 1.0)
  (set! us_rel_offset 0.0)
  (set! us_gain 0.9)

  (Parameter.set 'Synth_Method 'UniSyn)
  (Parameter.set 'us_sigpr 'lpc)
  (us_db_select itc_kh_po_db_name)

  ;; This is where you can modify power (and sampling rate) if desired
  (set! after_synth_hooks nil)
;  (set! after_synth_hooks
;      (list
;        (lambda (utt)
;          (utt.wave.rescale utt 2.1))))

  ;; set callback to restore some original values changed by this voice
  (set! current_voice_reset itc_kh_po_voice_reset)

  (set! current-voice 'itc_kh_po_diphone)
)

(proclaim_voice
 'itc_kh_po_diphone
 '((language english)
   (gender COMMENT)
   (dialect american)
   (description
    "COMMENT"
    )
   (builtwith festvox-1.3)))

(provide 'itc_kh_po_diphone)
