;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                     ;;;
;;;                     Carnegie Mellon University                      ;;;
;;;                  and Alan W Black and Kevin Lenzo                   ;;;
;;;                      Copyright (c) 1998-2000                        ;;;
;;;                        All Rights Reserved.                         ;;;
;;;                                                                     ;;;
;;; Permission is hereby granted, free of charge, to use and distribute ;;;
;;; this software and its documentation without restriction, including  ;;;
;;; without limitation the rights to use, copy, modify, merge, publish, ;;;
;;; distribute, sublicense, and/or sell copies of this work, and to     ;;;
;;; permit persons to whom this work is furnished to do so, subject to  ;;;
;;; the following conditions:                                           ;;;
;;;  1. The code must retain the above copyright notice, this list of   ;;;
;;;     conditions and the following disclaimer.                        ;;;
;;;  2. Any modifications must be clearly marked as such.               ;;;
;;;  3. Original authors' names are not deleted.                        ;;;
;;;  4. The authors' names are not used to endorse or promote products  ;;;
;;;     derived from this software without specific prior written       ;;;
;;;     permission.                                                     ;;;
;;;                                                                     ;;;
;;; CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK        ;;;
;;; DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     ;;;
;;; ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT  ;;;
;;; SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE     ;;;
;;; FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES   ;;;
;;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN  ;;;
;;; AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,         ;;;
;;; ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF      ;;;
;;; THIS SOFTWARE.                                                      ;;;
;;;                                                                     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Tokenizer for kh
;;;
;;;  To share this among voices you need to promote this file to
;;;  to say festival/lib/itc_kh/ so others can use it.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Load any other required files

;; Punctuation for the particular language
(set! itc_kh_po::token.punctuation "\"'`,:;!?{}[]")
(set! itc_kh_po::token.prepunctuation "\"'`{[")
(set! itc_kh_po::token.whitespace " \t\n\r")
(set! itc_kh_po::token.singlecharsymbols "<>")

;;; Voice/kh token_to_word rules 
(define (itc_kh_po::token_to_words token name)
  "(itc_kh_po::token_to_words token name)
Specific token to word rules for the voice itc_kh_po.  Returns a list
of words that expand given token with name."
  (set! tmp_name "")	
  (set! tmp_name (string-append tmp_name (to_english_digits name)))
  (set! name tmp_name)
  
  (set! km_cons "ក\\|ខ\\|គ\\|ឃ\\|ង\\|ច\\|ឆ\\|ជ\\|ឈ\\|ញ\\|ដ\\|ឋ\\|ឌ\\|ឍ\\|ណ\\|ត\\|ថ\\|ទ\\|ធ\\|ន\\|ប\\|ផ\\|ព\\|ភ\\|ម\\|យ\\|រ\\|ល\\|វ\\|ស\\|ហ\\|ឡ\\|អ")
  (set! km_vow "ា\\|ិ\\|ី\\|ឹ\\|ឺ\\|ុ\\|ូ\\|ួ\\|ើ\\|ឿ\\|ៀ\\|េ\\|ែ\\|ៃ\\|ោ\\|ៅ\\|ុំ\\|ំ\\|ាំ\\|ះ\\|ិះ\\|ឹះ\\|ែះ\\|ុះ\\|េះ\\|ោះ")
  (set! km_in "ឫ\\|ឫ\\|ឭ\\|ឮ\\|ឥ\\|ឦ\\|ឩ\\|ឪ\\|ឯ\\|ឰ\\|ឱ\\|ឳ")
  (cond   
    
   ((equal? (string-length name) 0)
   	nil
   )

   ((string-matches name "​\\|”\\|“") ;Zero space symbole is not read
   	nil
   )   
   
   ((string-matches name "\\(៖\\)")
   	(itc_kh_po::token_to_words token ":")
   )
   
   ((string-matches name (string-append ".*\\(" km_vow "\\)ៈ"))
   	(append (list (string-append (string-before name "ៈ"))))
   )
   
   ((string-matches name (string-append "[0-9]+""ៈ"))
   	(append (itc_kh_po::token_to_words token (string-before name "ៈ")) '("."))
   )
   
   ((string-matches name "\។")
   	(item.set_feat token "name" ".")
   	nil
   )
   
   ((string-matches name "\([1-2][0-9][0-9][0-9]\)")   	
   	(append '("ឆ្នាំ") (itc_kh::number (string-before (string-after name "(") ")")))
   )
   
   ((string-matches name "\(.+\)")   	   	
   	(append (append (list "បើក" "វង់" "ក្រចក") (itc_kh_po::token_to_words token (string-before (string-after name "(") ")")))
		(list "បិទ" "វង់" "ក្រចក"))   	
   )
   
   ((string-matches name "\(.+")   	
   	(item.set_feat token "name" (string-after name "(")) 
   	(append (list "បើក" "វង់" "ក្រចក") (itc_kh_po::token_to_words token (string-after name "(")))
   )
   
   ((string-matches name ".+\)")
   	(item.set_feat token "name" (string-before name ")"))
   	(append (itc_kh_po::token_to_words token (string-before name ")")) (list "បិទ" "វង់" "ក្រចក"))   	
   )   
    	;;Telephone 012216942
   ((string-matches name "0[1-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]?[0-9]?[0-9]?")
   	(append (itc_kh::number "0")(read_two_digit (string-after name 0)))
   )
   
      ;;+85512216942 We consider that the code zip in this case is thee digits
   ((string-matches name "\\(\\+[0-9][0-9]?[0-9]?\\)[1-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]?[0-9]?[0-9]?")
	(append '("បូក")		
		(read_two_digit (string-append "0" (string-after name "+")))
	)
   )            
   
   ;code zip and telephone number
   ((string-matches name "\\(\\+[0-9][0-9]?[0-9]?\\)")
   	(cond ((equal? (length name) 3)
			(append '("បូក")(read_two_digit (string-after name "+")))
		)
		(t
			(cond ((equal? (string-length (string-after name "+")) 3)
				(append '("បូក")(read_two_digit (string-append "0" (string-after name "+")))))
			(t
				(append '("បូក")(read_two_digit (string-after name "+"))))
			)	
		))
   )   
   ;adress IP
   ((string-matches name "[1-2]?[0-9]?[0-9]\\.[1-2]?[0-9]?[0-9]\\.[1-2]?[0-9]?[0-9]\\.[1-2]?[0-9]?[0-9]")   	
   	(set! toReturnIP '())
   	(set! toReturnIP (append (itc_kh_po::token_to_words token (string-before name ".")) '("ចុច")))
   	(set! name (string-after name "."))
   	(set! toReturnIP (append (append toReturnIP (itc_kh_po::token_to_words token (string-before name "."))) '("ចុច")))
   	(set! name (string-after name "."))
   	(set! toReturnIP (append (append toReturnIP (itc_kh_po::token_to_words token (string-before name "."))) '("ចុច")))
   	(set! name (string-after name "."))
   	(set! toReturnIP (append toReturnIP (itc_kh_po::token_to_words token name)))
   	toReturnIP
   )
   
   	;;number start with 0 it should pronounce one by one
   ((string-matches name "0[0-9]*")   
   	(set! toReturn '())
   	(set! tmp token)
   	(set! number "")
   	(while (and (item.next tmp nil) (string-matches (item.feat tmp "n.name") "[0-9][0-9]+"))
   		(set! number (string-append number (item.feat tmp "name")))
   		(set! tmp (item.next tmp))
   	)   	
   	(cond ((string-matches (item.feat tmp "name") "[0-9][0-9]+")
   		(set! number (string-append number (item.feat tmp "name")))
   	))
   	
   	(cond ((string-matches number "0[1-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]?[0-9]?[0-9]?")   	
   			(set! toReturn (append (list "សូន្យ") (read_two_digit (string-after name 0)))))   			   			
   		((equal? (item.feat token "p.name") "ខែ")
  	       	  (set! toReturn (cdr(assoc_string (remove_leading_zeros name) itc_kh_month))))  		
   		(t
   			(set! toReturn (itc_kh::read_one_digit name)))
   	)  	
   	toReturn   	
   )  

   ((string-matches name "-[0-9.,]+")
   	(append '("ដក") (itc_kh_po::token_to_words token (string-after name "-")))
   )
   
   ((string-matches name "\\+.*")
   	(append '("បូក") (itc_kh_po::token_to_words token (string-after name "+")))
   )
   
     	;;normal number 
   ((string-matches name "[1-9][0-9]*")
	;(itc_kh::number name)	
	;testing in case of time
	(set! toReturn '())
	(cond ((string-matches (downcase (item.feat token "n.name")) "\\(am\\)\\|\\(pm\\)")
			(cond ((equal? (item.feat token "p.name") "ម៉ោង")
				(set! toReturn (itc_kh::number name)))  			      
				(t
				(set! toReturn (append '("ម៉ោង") (itc_kh::number name))))
			)
		)
		((equal? (item.feat token "p.name") "ខែ")
  	       	  (set! toReturn (cdr(assoc_string (remove_leading_zeros name) itc_kh_month)))
  		)
  		((string-matches (item.feat token "p.name") "មករា\\|កុម្ភៈ\\|មិនា\\|មេសា\\|ឩសភា\\|មិថុនា\\|កកដ្ឋា\\|សីហា\\|កញ្ញា\\|តុលា\\|វិច្ឆកា\\|ធ្នូ")
  			(set! toReturn (append '("ឆ្នាំ") (itc_kh::number name)))
  		)
		(t
			(set! toReturn (itc_kh::number name)))
	)
	toReturn
   )
   
   ((string-matches name "មករា\\|កុម្ភៈ\\|មិនា\\|មេសា\\|ឩសភា\\|មិថុនា\\|កកដ្ឋា\\|សីហា\\|កញ្ញា\\|តុលា\\|វិច្ឆកា\\|ធ្នូ")
   	(cond ((and (not (string-matches (item.feat token "p.name") "ខែ")) 
   		(string-matches (to_english_digits (item.feat token "n.name")) "\\([1-2][0-9][0-9][0-9]\\(.\\)?\\)"))
      		   (append (list "ខែ" name)))
   		(t 
   		   (list name))
   	)
   )

	;;number with point
   ((string-matches name "[1-9][0-9]?[0-9]?\\(\\.[0-9][0-9][0-9]\\)*")
	(set! n "")   
   	(set! tmp name)
   	(while (>(string-length tmp) 3)
   		(set! n (string-append n (string-before tmp ".")))
   		(set! tmp (string-after tmp "."))
	)
	(set! n (string-append n tmp))	
	(itc_kh::number n)   	
   )  

    ((string-matches name ".*\\(\\,[0-9]+\\)")
    	(set! toReturn '())
	(set! toReturn (append (append (itc_kh_po::token_to_words token (string-before name ",")) '("ក្បៀស"))))
	(set! flag t)
	(set! tmp (string-after name ","))
	(while flag
		(cond ((string-matches tmp "0[0-9]*")
			(set! tmp (string-after tmp "0"))
			(set! toReturn (append toReturn '("សូន្យ"))))
			(t
				(set! flag (not t))
				(set! toReturn (append toReturn (itc_kh::number tmp))))
		)		
	)
	toReturn
    )
    
    ;;fraction2
    ((string-matches name "[1-9]+/[0-9]+")
    	(append (append (itc_kh::number (string-before name "/")) '("ភាគ")) 
		(itc_kh::number (string-after name "/")))	
    )
        
    ;;Time 12:12:13 (it does not work well)
    ((string-matches name "[0-2]?[0-9]:[0-6][0-9]?:[0-6][0-9]?")
        	
    	(set! toReturn '())    	
    	(cond ((not (equal? (item.feat token "p.name") "ម៉ោង"))    		
	    	(set! toReturn (append toReturn (list "ម៉ោង")))
 	))
 	
 	(append (append toReturn (itc_kh::number (string-before name ":")))
    			(append (itc_kh::number (string-before (string-after name ":") ":")) '("នាទី"))
    			(append (itc_kh::number (string-after (string-after name ":") ":")) '("វិនាទី")))
    )
    
    ;; Hours:Minutes
    ((string-matches name "[0-2]?[0-9]:[0-6][0-9]?")
    	(set! toReturn '())
    	(cond ((not (equal? (item.feat token "p.name") "ម៉ោង"))
  		(set! toReturn (append toReturn (list "ម៉ោង")))
    	))
    	(cond ((equal? (item.feat token "n.name") "នាទី")    	
	    	(append (append toReturn (itc_kh::number (string-before name ":"))) (itc_kh::number (string-after name ":")))
	    	)
    	    (t
    		(append (append toReturn (itc_kh::number (string-before name ":"))) 
    			(append (itc_kh::number (string-after name ":")) '("នាទី")))))
    )   
    
    ;; 8am
    ((string-matches (downcase name) "\\([0-1][0-2]\\|[1-9]\\)\\(am\\)")
    	(set! toReturn '())
    	(cond ((not (equal? (item.feat token "p.name") "ម៉ោង"))
  		(set! toReturn (append toReturn (list "ម៉ោង")))
    	))
    	(append (append toReturn (itc_kh::number (string-before (downcase name) "a"))) '("ព្រឹក"))
    )
    
    ;; 8pm
    ((string-matches (downcase name) "\\([0-1][0-2]\\|[1-9]\\)\\(pm\\)")    
    	(set! toReturn '())
    	(cond ((not (equal? (item.feat token "p.name") "ម៉ោង"))
  		(set! toReturn (append toReturn (list "ម៉ោង")))
    	))
    	
    	(append (append toReturn (itc_kh::number (string-before (downcase name) "p"))) '("ល្ងាច"))
     )    
    
     ((string-matches (downcase name) ".*am")
   	(set! toReturn '())
   	(cond ((eq (string-length (string-before (downcase name) "a")) 0)
   		(set! toReturn '("ព្រឺក")))
   		(t
   			(set! toReturn (append (itc_kh_po::token_to_words token (string-before (downcase name) "a")) '("ព្រឺក")))
   		)
   	)
   	toReturn
     )
   
     ((string-matches (downcase name) ".*pm")
	(set! toReturn '())
   	(cond ((eq (string-length (string-before (downcase name) "p")) 0)
   		(set! toReturn '("ល្ងាច")))
   		(t
   			(set! toReturn (append (itc_kh_po::token_to_words token (string-before (downcase name) "p")) '("ល្ងាច")))
   		)
   	)
   	toReturn
     )   
    
    ;;street normally they are seperated
        
    ;;Roman number
    ((string-matches name "\\(II?I?\\|IV\\|VI?I?I?\\|IX\\|X[VIX]*\\)")
    	(append (itc_kh_po::token_to_words token (tok_roman_to_numstring name)))
    )
    
    ;;to do it later 
    ;;Money
    ((string-matches name "\\$\\(.*\\)")
    	(append (itc_kh_po::token_to_words token (string-after name "$")) '("ដុល្លា"))
    )
    ;; to update later 
    ((string-matches name "\\(.*\\)EUR")    
    	(append (itc_kh_po::token_to_words token (string-before name "E")) '("អ៊ឺរ៉ូ"))
    )    
    
    	;;Normal number %
    ((string-matches name ".*%")
    	(append (itc_kh_po::token_to_words token (string-before name "%")) '("ភាគរយ"))
    )
            
    ;;Range 1-7
    ((string-matches name "[1-9][0-9]*\\-[1-9][0-9]*")
    	(append (append (append '("ពី") (itc_kh::number (string-before name "-"))) 
    		'("ទៅ")) (itc_kh::number (string-after name "-")))
    )
    ((string-matches name ".+-.*")
    	(append (itc_kh_po::token_to_words token (string-before name "-"))
    		(itc_kh_po::token_to_words token (string-after name "-")))
    )
    
    ;;Range 1~7
    ((string-matches name "[1-9][0-9]*\\~[1-9][0-9]*")
       	(append (append (append '("ពីរ") (itc_kh::number (string-before name "~"))) 
    		'("ទៅ")) (itc_kh::number (string-after name "~")))
    )    
    
    ((string-matches name "&")
	(list "និង")
    )
    
    ((string-matches name ".+&.+")
    	(append (append (itc_kh_po::token_to_words token (string-before name "&")) (list "និង"))
    	(itc_kh_po::token_to_words token (string-after name "&")))
    )
    
    ((string-matches name "\|")
    	(cond ((and (item.prev token) (item.next token))
	   	(list "ឫ"))
	  (t
	  	nil)
	  )
    )
    
    ((string-matches name ".+\|.+")
    	(append (append (itc_kh_po::token_to_words token (string-before name "|")) (list "ឫ"))
	    	(itc_kh_po::token_to_words token (string-after name "|")))
    )
    
    ;;Latin
    ((string-matches name "[a-zA-Z@]+")
      (utf8explode (downcase name))
;    	(set! toReturn '())
;	(let ((se (utf8explode (downcase name))))
;    		(while (> (length se) 0)
;    			(set! toReturn (append toReturn (car (cdr (assoc_string (car se) itc_kh_list_latin)))))
;    			(set! se (cdr se))
;    		)
;    	)
;    	toReturn  
    )
    
    ((string-matches name "\\([0-9]+\\)\\.\\([0-9]+\\)")
    	(append (itc_kh::read_one_digit (string-before name "."))
    		 '("ចុច")  (itc_kh::read_one_digit (string-after name ".")))
    )
    
    ;; Numero and Latine    
    ((string-matches name "[1-9][0-9]+[a-zA-Z]+")
    	(append (itc_kh_po::token_to_words token (car (itc_kh::split_num_al name)))    	
    		(itc_kh_po::token_to_words token (car (cdr (itc_kh::split_num_al name))))
    	)    	
    )
        
    ;;lek TO
    ((string-matches name ".*ៗ")
    	(cond ((string-matches name "ៗ")
    		(cond ((equal? (item.prev token) nil)
    			(list "លេខ" "ទ"))
    		   (t
    		   	(list (item.feat token "p.name"))
    		   ))
    	)
    	
    	(t
    	   (append (itc_kh_po::token_to_words token (string-before name "ៗ"))
	    	(itc_kh_po::token_to_words token (string-before name "ៗ"))
	)))    	
    )
    
    ;;the abreviation and acronim is end with .
    ((string-matches name (string-append "\\([" km_cons "\\|" km_in "\\|" km_vow "]+\\.\\)+"))
    	(car (cdr (assoc_string name itc_kh_list_abv)))   
    )    
    
     ;;Identifier 
    ((string-matches name (string-append "\\([0-9]+\\)?[" km_cons "]\\.[" km_cons "]"))
    	(set! toReturn (itc_kh::split_num_al name))
    	(set! t_num (car toReturn))
    	(set! t_alp (car (cdr toReturn)))
    	(set! toReturn '())
    	(set! tmp (string-append (string-before t_alp ".") (string-after t_alp ".")))
    	(let ((se (utf8explode tmp)))
    		(while (> (length se) 0)
    			(set! toReturn (append toReturn (list (car se) ",")))
    			(set! se (cdr se))
    		)
    	)
    	(append (itc_kh::read_one_digit t_num) toReturn)
    )  
  
    ;;Date day/month/year(2 or 4)
    ((string-matches name "[0-9][0-9]?/[0-9][0-9]?/[0-9][0-9]\\([0-9][0-9]\\)?")
	(let ((num1 (string-before name "/"))
	     (num2 (string-before (string-after name "/") "/"))
	     (year (string-after (string-after name "/") "/"))
	        day month)
	     (set! day (itc_kh::eigth_digits_number num1))
	     (set! day (append (list "ថ្ងៃ" "ទី") day))
	     
	     ;month have to be tested
	     (set! month (cdr(assoc_string (remove_leading_zeros num2) itc_kh_month)))
	     (set! month (append (list "ខែ") month))
	     
             ;;year	     
             (set! year (read_year year))
	     (set! year (append (list "ឆ្នាំ") year))
	   (append (append day month) year)
	))
	;;signe
    ((string-matches name "\\+\\|-\\|\\*\\|/\\|=")
    	(set! toReturn '())
    	(cond ((string-matches name "-")
    		(cond ((and (string-matches (to_english_digits (item.feat token "p.name")) "[1-2][0-9][0-9][0-9]") 
    			(not (string-matches (to_english_digits (item.feat token "n.name")) "[0-9]+")))   			
    			(set! toReturn '("រហូត" "ដល់")))
    		  ((and (string-matches (to_english_digits (item.feat token "p.name")) "[0-9]+")
    			(string-matches (to_english_digits (item.feat token "n.name")) "[0-9]+"))
    			(set! toReturn (car (cdr (assoc_string name itc_kh_signe)))))    			
    		  (t 
       		    nil)
       		 )        		
       	   )
    	   (t    	   	
    		(set! toReturn (car (cdr (assoc_string name itc_kh_signe))))
    	   ))
    	   toReturn
    )
    
    ;; These symbole should not read
    ((string-matches name "_____+")
     nil)
    ((string-matches name "-----+")
     nil)
    ((string-matches name "=====+")
     nil)
    ((string-matches name "\\*\\*\\*\\*\\*+")
     nil)    
     
     ; Read symbol
    ((string-matches name "\(")
    	(cond ((equal? (string-length (item.feat token "name")) 1)
    		(list ","))
    	(t
    		(list "បើក" "វង់" "ក្រចក")
    	))
    )
    
    ((string-matches name "\)")
    	(cond ((equal? (string-length (item.feat token "name")) 1)
    		(list ","))
    	(t
    		(list "បិទ" "វង់" "ក្រចក")
    	))
    )
    
    ((string-matches name "\\.")
    	(list "ចុច")
    )
        	
   (t
   ;	;the NSW can not read so we read it one by one
   	(cond ((string-matches name "\\(.*\\)[\\.\(]\\(.*\\)")  ;to define later
   		(set! unknow_word '())
   		(let ((se (utf8explode name)))
   			(while (> (length se) 0)
   				(set! tmp (car se))
   				(set! unknow_word (append unknow_word (itc_kh_po::token_to_words token tmp)))
   				(set! se (cdr se))
   			)
   		)
   		unknow_word)
   	(t
   		;; It should read the metric
   	   (set! m_tmp '())   	   
     	   (set! m_tmp (cdr (assoc_string name itc_kh_list_metric)))
     	   (cond ((and (not (equal? m_tmp nil)) (string-matches (item.feat token "p.name") "[0-9.,]+"))
		(car m_tmp)
     	   )
     	   (t
   	   	(list name))
   	   )
   	))
   ))
   
)

(define (itc_kh_po::select_tokenizer)
  "(itc_kh_po::select_tokenizer)
Set up tokenizer for kh."
  (Parameter.set 'Language 'itc_kh)
  (set! token.punctuation itc_kh_po::token.punctuation)
  (set! token.prepunctuation itc_kh_po::token.prepunctuation)
  (set! token.whitespace itc_kh_po::token.whitespace)
  (set! token.singlecharsymbols itc_kh_po::token.singlecharsymbols)

  (set! token_to_words itc_kh_po::token_to_words)
  (set! itc_kh_digit_names
	'((0 "សូន្យ")
  	 ( 1 "មួយ")
	 (2 "ពីរ")
	 (3 "បី")
	 (4 "បួន")
	 (5 "ប្រាំ")
	 (6 "ប្រាំមួយ")
	 (7 "ប្រាំពីរ")
	 (8 "ប្រាំបី")
	 (9 "ប្រាំបួន")))
   (set! itc_kh_two_digits
	'((1 "ដប់")
	(2 "ម្ភៃ")
	 (3 "សាមសិប")
	 (4 "សែសិប")
	 (5 "ហាសិប")
	 (6 "ហុកសិប")
	 (7 "ចិតសិប")
	 (8 "ប៉ែតសិប")
	 (9 "កៅសិប")))
   (set! itc_kh_level_digits
	'((3 "រយ")
	(4 "ពាន់")
	(5 "ម៉ឺន")	
	(6 "សែន")
	(7 "លាន")
	(8 "កោដ")))
   (set! itc_kh_month
	'((1 "មករា")
	  (2 "កុម្ភៈ")
	  (3 "មិនា")
	  (4 "មេសា")
    	  (5 "ឩសភា")
	  (6 "មិថុនា")
	  (7 "កកដ្ឋា")
	  (8 "សីហា")
	  (9 "កញ្ញា")
	  (10 "តុលា")
	  (11 "វិច្ឆកា")
	  (12 "ធ្នូ")))
	  ;;to add the abreviation later
   (set! itc_kh_list_abv	
	'(("វ.ប.ក." ("វិទ្យាស្ថាន" "បច្ចេកវិទ្យា" "កម្ពុជា"))		
	("គ.ជ.ប." ("គណៈ" "កម្មការ" "ជាតិ" "រាបចំ" "ការ" "បោះឆ្នោត"))
	("ស.ច.ណ." ("សារាចរ" "ណែនាំ"))
	("ស.ក." ("សង្កាត់"))
	("ឯ." ("ឯឧត្ដម"))		
	("នា." ("នាម"))
	("កិ." ("កិរិយាស័ព្ទ"))
	("រ.វ." ("រាជវាំង"))
	("វេជ្ជ." ("វេជ្ជបណ្ឌិត"))
	))  
	
   (set! itc_kh_list_latin
	'(("a" ("អា"))		
	("b" ("បេ"))
	("c" ("សេ"))
	("d" ("ដេ"))
	("e" ("អ៊ឺ"))
	("f" ("អ៊ែវហ្វ"))
	("g" ("ហ្ស៊េ"))
	("h" ("អះ"))
	("i" ("អ៊ី"))
	("j" ("ហ្សី"))
	("k" ("កា"))
	("l" ("អែល"))
	("m" ("អឺម"))
	("n" ("អ៊ិន"))
	("o" ("អ៊ូ"))
	("p" ("ប៉េ"))
	("q" ("គុយ"))
	("r" ("អ៊ែរ"))
	("s" ("អេស"))
	("t" ("តេ"))
	("u" ("អ៊ុយ"))
	("v" ("វេ"))
	("w" ("ឌុបលឺវេ"))
	("x" ("អិក"))
	("y" ("អ៊ីការិក"))
	("z" ("ហ្សិត"))
	("@" ("អារ៉ូបា"))
	))
	(set! itc_kh_signe
	'(("+" ("បូក" "នឹង"))
   	  ("*" ("គុណ" "អោយ"))
	  ("/" ("ចែក" "នឹង"))
	  ("=" ("ស្មើរ"))
	  ("(" ("បើក" "វង់" "ក្រចក"))
	  ("-" ("ដក" "អោយ"))))
	  
	(set! itc_kh_list_metric
	'(("ម" ("ម៉ែត"))   	  
	("ម2" ("ម៉ែត" "ការេ"))
	  ("ល" ("លីត្រ"))))
)

(define (itc_kh::number name)
  "(itc_kh::number name)
Return list of words that pronounce this number in kh."
   
	(let ((se (reverse (utf8explode name))))	
	(set! tmp "")
	(set! toReturn '())
	(set! tmp_list '())
	(set! index_i 1)
	(while (> (length se) 0)
	   (set! tmp (string-append (car se) tmp))
	   (set! index_i (+ index_i 1))	
           (set! se (cdr se))		
	   (cond ((equal? index_i 8)		
		(set! tmp_list (append (list tmp) tmp_list))
		(set! index_i 0)
		(set! tmp "")
	   ))
	))
	;;the n first digits
	(cond ((> index_i 0)
		(set! tmp_list (append (list tmp) tmp_list))
	))

	(while (> (length tmp_list) 1)
		(set! toReturn (append toReturn (itc_kh::eigth_digits_number (car tmp_list))))
		(set! toReturn (append toReturn (list "កោដ")))
		(set! tmp_list (cdr tmp_list))
	)	
	(set! toReturn (append toReturn (itc_kh::eigth_digits_number (car tmp_list))))    
	
	(cond ((string-matches name "[0]+")
		(set! toReturn '("សូន្យ"))
	))	
   toReturn;
)

(define (itc_kh::eigth_digits_number digits)
"read the sequece of 8 digits of the number
digits is a string"	
    (let ((se (utf8explode digits)))
    (set! toReturn '())    
    (set! index_i (length se))
    (while (> (length se) 0)	
	(cond ((not (string-equal (car se) "0"))
		(cond ((equal? index_i 1)
			(set! toReturn (append toReturn (list (car (cdr (assoc_string (car se) itc_kh_digit_names)))))))
		     ((equal? index_i 2)
			(set! toReturn (append toReturn (list (car (cdr (assoc_string (car se) itc_kh_two_digits)))))))
		     (t		
			(set! toReturn (append toReturn (list (car (cdr (assoc_string (car se) itc_kh_digit_names))))))
			(set! toReturn (append toReturn (list (car (cdr (assoc_string index_i itc_kh_level_digits)))))))
		))	
	)	
	(set! index_i (- index_i 1))
	(set! se (cdr se))
    ))
	toReturn;
)

(define (read_two_digit digits)
    (let ((se (utf8explode digits)))
    (set! result '())
    (set! tmp "")    
    	(while (> (length se) 1)    		    		    		
    		(set! tmp (car se))
    		(set! se (cdr se))
    		(set! tmp (string-append tmp (car se)))
    		(set! se (cdr se))    		
    		
    		(cond ((equal? (length se) 1)
    			(set! tmp (string-append tmp (car se)))
	    		(set! se (cdr se)))
    		)
    		(set! result (append result (itc_kh::number tmp)))
    	)   	
    )
    result
)

(define (itc_kh::read_one_digit digits)
"digits is a string"
	(mapcar
	 (lambda (d)
	  (car (cdr (assoc_string d itc_kh_digit_names))))
	(utf8explode digits))	
)

(define (read_year digits)	
	(set! toRead "")
	(cond ((string-matches digits "[0-2][0-9]")
		(set! toRead (string-append "20" digits)))
	      ((string-matches digits "[3-9][0-9]")
		(set! toRead (string-append "19" digits)))
	      (t
		(set! toRead digits))
	)	
	(itc_kh::eigth_digits_number toRead)
)

(define (to_tel_format number)
	(let ((se (utf8explode number)))
		
		
	)
)

(define (itc_kh_po::reset_tokenizer)
  "(itc_kh_po::reset_tokenizer)
Reset any globals modified for this voice.  Called by 
(itc_kh_po::voice_reset)."
  ;; None

  t
)

(define (to_english_digits digits)
	(let ((se (utf8explode digits)))
	(set! toReturnDigit "")
	   (while (> (length se) 0)		
	   	(set! tmp (car se))
		(cond ((string-equal tmp "០")
			(set! toReturnDigit (string-append toReturnDigit "0")))
		((string-equal tmp "១")
			(set! toReturnDigit (string-append toReturnDigit "1")))
		((string-equal tmp "២")
			(set! toReturnDigit (string-append toReturnDigit "2")))
		((string-equal tmp "៣")
			(set! toReturnDigit (string-append toReturnDigit "3")))
		((string-equal tmp "៤")
			(set! toReturnDigit (string-append toReturnDigit "4")))
		((string-equal tmp "៥")
			(set! toReturnDigit (string-append toReturnDigit "5")))
		((string-equal tmp "៦")
			(set! toReturnDigit (string-append toReturnDigit "6")))
		((string-equal tmp "៧")
			(set! toReturnDigit (string-append toReturnDigit "7")))
		((string-equal tmp "៨")
			(set! toReturnDigit (string-append toReturnDigit "8")))
		((string-equal tmp "៩")
			(set! toReturnDigit (string-append toReturnDigit "9")))
		(t
			(set! toReturnDigit (string-append toReturnDigit tmp)))
		)
		(set! se (cdr se))
	   )
	)	
	toReturnDigit
)

(define (itc_kh::split_num_al phr)
	(set! num "")
	(set! alp "")
	(set! flag t)
	(let ((se (utf8explode phr)))
	   (while (> (length se) 0)
		(cond ((and (string-matches (car se) "[0-9]") flag)
			(set! num (string-append num (car se))))
		     (t
		      	(set! flag (not t))
		      	(set! alp (string-append alp (car se)))
		     ))
		 (set! se (cdr se))
	    )  
	)
	(list num alp)
)

(define (remove_dot))

(define (just_zeros digits)
"(just_zeros digits)
If this only contains 0s then we just do something different."
 (cond
  ((not digits) t)
  ((string-equal "0" (car digits))
   (just_zeros (cdr digits)))
  (t nil))
)

(provide 'itc_kh_po_tokenizer)

